/**
 * RapidMiner GmbH
 *
 * Copyright (C) 2020 by RapidMiner GmbH and the contributors
 *
 * Complete list of developers available at our web site:
 *
 *      www.rapidminer.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package com.rapidminer.extension.shared_data.connection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.function.BiConsumer;

import org.apache.commons.lang.ArrayUtils;

import com.rapidminer.connection.ConnectionHandler;
import com.rapidminer.connection.ConnectionInformation;
import com.rapidminer.connection.ConnectionInformationBuilder;
import com.rapidminer.connection.configuration.ConfigurationParameter;
import com.rapidminer.connection.configuration.ConnectionConfiguration;
import com.rapidminer.connection.configuration.ConnectionConfigurationBuilder;
import com.rapidminer.connection.util.ParameterUtility;
import com.rapidminer.connection.util.TestExecutionContext;
import com.rapidminer.connection.util.TestResult;
import com.rapidminer.connection.util.ValidationResult;
import com.rapidminer.connection.valueprovider.handler.ValueProviderHandlerRegistry;
import com.rapidminer.extension.shared_data.PluginInitSharedData;
import com.rapidminer.operator.Operator;


/**
 * A {@link ConnectionHandler} for shared data connections.
 * <br>Parameters are:
 * <ul>
 *     <li>user: user name; mandatory</li>
 *     <li>password: password; encrypted, mandatory</li>
 *     <li>option: selection between A, B and C; default: B</li>
 *     <li>single_A: single parameter for option A; mandatory</li>
 *     <li>first_B: first parameter for option B; optional</li>
 *     <li>second_B: second parameter for Option B; optional</li>
 *     <li>use_for_C: boolean parameter for option C; mandatory, default: false</li>
 * </ul>
 *
 * @since 1.0.0
 */
public enum SharedDataHandler implements ConnectionHandler {
	INSTANCE;

	public static final String SHARED_DATA_TYPE_ID = "shared-data";
	public static final String SHARED_DATA_FULL_TYPE = PluginInitSharedData.NAMESPACE + ":" + SHARED_DATA_TYPE_ID;

	public static final String OPTION_A = "A";
	public static final String OPTION_B = "B";
	public static final String OPTION_C = "C";
	public static final String[] OPTIONS = {OPTION_A, OPTION_B, OPTION_C};

	public static final String GROUP_BASIC = "basic";
	public static final String PARAMETER_USER = "user";
	public static final String PARAMETER_PASSWORD = "password";
	public static final String PARAMETER_OPTION = "option";
	public static final String PARAMETER_SINGLE_A = "single_a";
	public static final String PARAMETER_FIRST_B = "first_b";
	public static final String PARAMETER_SECOND_B = "second_b";
	public static final String PARAMETER_USE_FOR_C = "use_for_c";
	public static final String VALIDATION_WRONG_VALUE = "validation.wrong_value";

	@Override
	public ConnectionInformation createNewConnectionInformation(String name) {
		ConnectionConfigurationBuilder configBuilder = new ConnectionConfigurationBuilder(name, SHARED_DATA_FULL_TYPE);
		ArrayList<ConfigurationParameter> parameters = new ArrayList<>();
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_USER).build());
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_PASSWORD, true).build());
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_OPTION).withValue(OPTION_B).build());
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_SINGLE_A).disable().build());
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_FIRST_B).build());
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_SECOND_B).build());
		parameters.add(ParameterUtility.getCPBuilder(PARAMETER_USE_FOR_C).disable().build());
		configBuilder.withKeys(GROUP_BASIC, parameters);
		return new ConnectionInformationBuilder(configBuilder.build()).build();
	}

	@Override
	public String getType() {
		return SHARED_DATA_FULL_TYPE;
	}

	@Override
	public ValidationResult validate(ConnectionInformation connection) {
		if (connection == null) {
			return ValidationResult.nullable();
		}
		ConnectionConfiguration configuration = connection.getConfiguration();
		if (configuration == null) {
			return ValidationResult.nullable();
		}
		Map<String, String> errors = new LinkedHashMap<>();
		BiConsumer<String, String> errorCollector = errors::put;
		ParameterUtility.validateParameterValue(GROUP_BASIC, PARAMETER_USER, configuration, errorCollector);
		ParameterUtility.validateParameterValue(GROUP_BASIC, PARAMETER_PASSWORD, configuration, errorCollector);
		ParameterUtility.validateParameterValue(GROUP_BASIC, PARAMETER_OPTION, configuration, errorCollector);
		String fullOptionKey = GROUP_BASIC + '.' + PARAMETER_OPTION;
		if (!errors.containsKey(fullOptionKey)) {
			String optionValue = configuration.getValue(fullOptionKey);
			if (!configuration.getParameter(fullOptionKey).isInjected() && !ArrayUtils.contains(OPTIONS, optionValue)) {
				errors.put(fullOptionKey, VALIDATION_WRONG_VALUE);
			} else {
				switch (optionValue) {
					case OPTION_A:
						ParameterUtility.validateParameterValue(GROUP_BASIC, PARAMETER_SINGLE_A, configuration, errorCollector);
						break;
					case OPTION_C:
						ParameterUtility.validateParameterValue(GROUP_BASIC, PARAMETER_USE_FOR_C, configuration, errorCollector);
						String fullUseCKey = GROUP_BASIC + '.' + PARAMETER_USE_FOR_C;
						if (!errors.containsKey(fullUseCKey)) {
							String useCValue = configuration.getValue(fullUseCKey);
							if (!configuration.getParameter(fullUseCKey).isInjected()
									&& !useCValue.equalsIgnoreCase(Boolean.FALSE.toString())
									&& !useCValue.equalsIgnoreCase(Boolean.TRUE.toString())) {
								errors.put(fullUseCKey, VALIDATION_WRONG_VALUE);
							}
						}
						break;
					default:
						break;
				}
			}
		}
		return errors.isEmpty() ?
				ValidationResult.success(ValidationResult.I18N_KEY_SUCCESS) :
				ValidationResult.failure(ValidationResult.I18N_KEY_FAILURE, errors);
	}

	@Override
	public TestResult test(TestExecutionContext<ConnectionInformation> testContext) {
		return TestResult.success(TestResult.I18N_KEY_NOT_IMPLEMENTED);
	}

	@Override
	public void initialize() {
		// noop
	}

	@Override
	public boolean isInitialized() {
		return true;
	}

	public static Map<String, String> getConfiguration(ConnectionInformation connection, Operator operator) {
		Map<String, String> values = ValueProviderHandlerRegistry.getInstance().injectValues(connection, operator, false);
		Map<String, String> configuration = new HashMap<>();
		configuration.put(PARAMETER_USER, values.get(GROUP_BASIC + '.' + PARAMETER_USER));
		switch (values.get(GROUP_BASIC + '.' + PARAMETER_OPTION)) {
			case OPTION_A:
				configuration.put(PARAMETER_SINGLE_A, values.get(GROUP_BASIC + '.' + PARAMETER_SINGLE_A));
				break;
			case OPTION_B:
				configuration.put(PARAMETER_FIRST_B, values.get(GROUP_BASIC + '.' + PARAMETER_FIRST_B));
				configuration.put(PARAMETER_SECOND_B, values.get(GROUP_BASIC + '.' + PARAMETER_SECOND_B));
				break;
			case OPTION_C:
				configuration.put(PARAMETER_USE_FOR_C, values.get(GROUP_BASIC + '.' + PARAMETER_USE_FOR_C));
				break;
		}
		return configuration;
	}
}
